require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # App deployment.
    class AppDeployment < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/AppDeployment"

      # Creates a new instance of +AppDeployment+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(resource_pool_uri, attributes)
      #   Creates a new deployment.
      #   @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      #   @param attributes [Hash] Application attributes.
      #   @option attributes [String] name Application name. May be +nil+.
      #   @option attributes [String] code Application code. May be +nil+.
      #   @option attributes [String] asid uuAppServer ID. May be +nil+.
      #   @option attributes [String] description Application description.
      #     May be +nil+.
      #   @option attributes [UU::OS::UESURI] appBoxUri AppBox URI.
      #   @option attributes [String] state Application state. Value must be
      #     one from {UU::OperationRegistry::AppDeploymentState}. May be +nil+.
      #   @option attributes [String] version Version specified in uuApp
      #     deployment descriptor. May be +nil+.
      #   @option attributes [String] urlPath URL path (+/vendor-app+)
      #     specified in uuApp deployment descriptor.
      #   @option attributes [TrueClass, FalseClass] multitenant Value of the
      #     +multitenant+ attribute specified in uuApp deployment descriptor.
      #     May be +nil+.
      #   @option attributes [String] config JSON string containing copy of
      #     important data from uuApp deployment descriptor and deployment-time
      #     configuration for the +PRODUCTION+ slot.
      #     These information should allow to redeploy the
      #     app when the corresponding uuApp Box does not exist. May be +nil+.
      #   @option attributes [String] appBoxCode AppBox code. Used when AppBox
      #     can no longer be addressed by +appBoxUri+ (i.e. the AppBox artifact
      #     has been deleted). If an artifact with this code exists, it will
      #     used instead. May be +nil+.
      #   @option attributes [Hash] slotConfig Keys are lowercase names of
      #     slots (e.g. +beta+ for the +BETA+ slot) and values are JSON strings
      #     containing copy of important data from uuApp deployment descriptor
      #     and deployment-time configuration. These information should allow
      #     to redeploy the app when the corresponding uuApp Box does not exist.
      #     Config for +PRODUCTION+ slot is in the +config+ attribute, see above.
      #     +NONE+ is not a regular slot and cannot be specified in +slotConfig+.
      #     May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the created deployment.

      # @!method self.create(resource_pool_uri, attributes)
      #   Creates a new deployment. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      #   @param attributes [Hash] Application attributes.
      #   @return [UU::OS::UESURI] UESURI of the created deployment.

      # @!method get_attributes(app_deployment)
      #   Returns attributes of the deployment specified by deployment URI
      #   or +tid-asid+ string and returns its detail.
      #   @param app_deployment [String, UU::OS::UESURI] Application URI or
      #     +tid-asid+ string.
      #   @return [Hash] Application attributes.

      # @!method self.get_attributes(app_deployment)
      #   Returns attributes of the deployment specified by deployment URI
      #   or +tid-asid+ string and returns its detail. This method is a
      #   convenience shortcut for the {#get_attributes} instance method.
      #   @param app_deployment [String, UU::OS::UESURI] Application URI or
      #     +tid-asid+ string.
      #   @return [Hash] Application attributes.

      # @!method set_attributes(app_deployment_uri, attributes)
      #   Sets attributes of an deployment.
      #   @param app_deployment_uri [String, UU::OS::UESURI] Application URI.
      #   @param attributes [Hash] Application attributes.
      #   @option attributes [String] name Application name. May be +nil+.
      #   @option attributes [String] code Application code. May be +nil+.
      #   @option attributes [String] asid uuAppServer ID. May be +nil+.
      #   @option attributes [String] description Application description.
      #     May be +nil+.
      #   @option attributes [String] state Application state. Value must be
      #     one from {UU::OperationRegistry::AppDeploymentState}. May be +nil+.
      #   @option attributes [String] runningProcess Identification of the currently
      #     running asynchronous process modifying this deployment. May be +nil+.
      #   @option attributes [String] version Version specified in uuApp
      #     deployment descriptor. May be +nil+.
      #   @option attributes [String] urlPath URL path (+/vendor-app+)
      #     specified in uuApp deployment descriptor. May be +nil+.
      #   @option attributes [TrueClass, FalseClass] multitenant Value of the
      #     +multitenant+ attribute specified in uuApp deployment descriptor.
      #     May be +nil+.
      #   @option attributes [String] config JSON string containing copy of
      #     important data from uuApp deployment descriptor and deployment-time
      #     configuration for the +PRODUCTION+ slot.
      #     These information should allow to redeploy the
      #     app when the corresponding uuApp Box does not exist. May be +nil+.
      #   @option attributes [Hash] slotConfig Keys are lowercase names of
      #     slots (e.g. +beta+ for the +BETA+ slot) and values are JSON strings
      #     containing copy of important data from uuApp deployment descriptor
      #     and deployment-time configuration. These information should allow
      #     to redeploy the app when the corresponding uuApp Box does not exist.
      #     Config for +PRODUCTION+ slot is in the +config+ attribute, see above.
      #     +NONE+ is not a regular slot and cannot be specified in +slotConfig+.
      #     May be +nil+.
      #   @option attributes [String] appBoxCode AppBox code. Used when AppBox
      #     can no longer be addressed by +appBoxUri+ (i.e. the AppBox artifact
      #     has been deleted). If an artifact with this code exists, it will
      #     used instead. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the updated deployment.

      # @!method self.set_attributes(app_deployment_uri, attributes)
      #   Sets attributes of an deployment. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param app_deployment_uri [String, UU::OS::UESURI] Application URI.
      #   @return [UU::OS::UESURI] UESURI of the updated deployment.

      # @!method delete(app_deployment_uri)
      #   Deletes the specified deployment from the registry.
      #   @param app_deployment_uri [String, UU::OS::UESURI] Application URI.
      #   @return [void]

      # @!method self.delete(app_deployment_uri)
      #   Deletes the specified deployment from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param app_deployment_uri [String, UU::OS::UESURI] Application URI.
      #   @return [void]

      # Returns a list of deployments in the given pool. Command does
      # not return all deployments, only a sublist. The list of returned deployments
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      # @param criteria [Hash] Criteria for listing deployments.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, appBoxUri, resourcePoolUri, tenantUri, state, urlPath. State must be one from {UU::OperationRegistry::AppDeploymentState}
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_app_deployment_list(resource_pool_uri, criteria = {})
        client.invoke(:getAppDeploymentList, resource_pool_uri, parameters: criteria)
      end

      # Returns list of deployments. This method is a
      # convenience shortcut for the {#get_app_deployment_list} instance method.
      #
      # @see #get_app_deployment_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_app_deployment_list(resource_pool_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_app_deployment_list(resource_pool_uri, criteria)
      end

    end

  end
end

