require 'uu/operation_registry'
require 'uu/os/cmd/command_client'
require 'uu/os/security/session'


module UU
  module OperationRegistry

    # Log of changes in capacity allocation.
    class CapacityLog

      # @private
      PATH = 'uu-operation-registry/CapacityLog'

      def initialize(session = nil)
        UU::OperationRegistry::BaseMethods.send(:_init_command_client, PATH)
        @client = UU::OS::CMD::CommandClient.new(PATH, session)
      end

      # Returns a list of capacity log records for given resource pool. The list of returned records
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      #
      # @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      # @param criteria [Hash] Criteria for listing resource pools.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_record_list(resource_pool_uri, criteria = {})
        @client.invoke(:getRecordList, resource_pool_uri, parameters: criteria)
      end

      # Returns a list of capacity log records for given resource pool. The list of returned records
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      #
      # @param resource_pool_uri [String, UU::OS::UESURI] Resource pool URI.
      # @param criteria [Hash] Criteria for listing resource pools.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_record_list(resource_pool_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_record_list(resource_pool_uri, criteria)
      end

      # Adds initial capacity record for all tenants and all resource pools for the specified month. This command
      # creates new capacity log record that represents initial of the particular resource pool capacity to the first
      # day of the specified month at 0:00:00.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param parameters [Hash] Parameters for consolidating resource pool capacity.
      # @option parameters [String] month Determines which month to add initial capacity for. Expected format: +yyyy-mm+.
      # @return [void]
      def consolidate(cloud_uri, parameters = {})
        @client.invoke(:consolidate, cloud_uri, parameters: parameters)
      end

      # Adds initial capacity record for all tenants and all resource pools for the specified month. This method is
      # a convenience shortcut for the {#consolidate} instance method.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param parameters [Hash] Parameters for consolidating resource pool capacity.
      # @return [void]
      def self.consolidate(cloud_uri, parameters = {})
        self.new(UU::OS::Security::Session.current_session).consolidate(cloud_uri, parameters)
      end
    end

  end
end
