require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Container for running applications.
    class Container < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/Container"

      # Creates a new instance of +Container+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(node_uri, attributes)
      #   Creates a new container.
      #   @param node_uri [String, UU::OS::UESURI] Node URI.
      #   @param attributes [Hash] Container attributes.
      #   @option attributes [String] name Container name.
      #   @option attributes [String] code Container code. May be +nil+.
      #   @option attributes [String] description Container description.
      #     May be +nil+.
      #   @option attributes [UU::OS::UESURI] nodeImageUri Node image URI.
      #   @option attributes [Array<Fixnum>] inboundPorts Inbound ports. May be +nil+.
      #   @option attributes [UU::OS::UESURI] nodeSetUri Node set URI.
      #   @option attributes [String] state Container state. Value must be
      #     one from {UU::OperationRegistry::ContainerState}. May be +nil+.
      #   @option attributes [UU::OS::UESURI] appDeploymentUri UESURI of
      #     deployment that this container belongs to. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the created container.

      # @!method self.create(node_uri, attributes)
      #   Creates a new container. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param node_uri [String, UU::OS::UESURI] Node URI.
      #   @param attributes [Hash] Container attributes.
      #   @return [UU::OS::UESURI] UESURI of the created container.

      # @!method get_attributes(container_uri)
      #   Gets the attributes of the container specified by container URI
      #   and returns its detail.
      #   @param container_uri [String, UU::OS::UESURI] Container URI.
      #   @return [Hash] Container attributes.

      # @!method self.get_attributes(container_uri)
      #   Gets the attributes of the container specified by container URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param container_uri [String, UU::OS::UESURI] Container URI.
      #   @return [Hash] Container attributes.

      # @!method set_attributes(container_uri, attributes)
      #   Sets attributes of a container.
      #   @param container_uri [String, UU::OS::UESURI] Container URI.
      #   @param attributes [Hash] Container attributes.
      #   @option attributes [String] name Container name. May be +nil+.
      #   @option attributes [String] code Container code. May be +nil+.
      #   @option attributes [String] description Container description.
      #     May be +nil+.
      #   @option attributes [Array<Fixnum>] inboundPorts Inbound ports. May be +nil+.
      #   @option attributes [String] state Container state. Value must be
      #     one from {UU::OperationRegistry::ContainerState}. May be +nil+.
      #   @option attributes [UU::OS::UESURI] appDeploymentUri UESURI of
      #     deployment that this container belongs to. May be +nil+.
      #   @option attributes [UU::OS::UESURI] nodeSetUri Node set URI.
      #   @return [UU::OS::UESURI] UESURI of the updated container.

      # @!method self.set_attributes(container_uri, attributes)
      #   Sets attributes of a container. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param container_uri [String, UU::OS::UESURI] Container URI.
      #   @return [UU::OS::UESURI] UESURI of the updated container.

      # @!method delete(container_uri)
      #   Deletes the specified container from the registry.
      #   @param container_uri [String, UU::OS::UESURI] Container URI.
      #   @return [void]

      # @!method self.delete(container_uri)
      #   Deletes the specified container from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param container_uri [String, UU::OS::UESURI] Container URI.
      #   @return [void]

      # Returns a list of containers on the given node. Command does
      # not return all containers, only a sublist. The list of returned containers
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param node_uri [String, UU::OS::UESURI] Node URI.
      # @param criteria [Hash] Criteria for listing containers.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: name, code, nodeImageUri, nodeUri, nodeSetUri, state
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_container_list(node_uri, criteria = {})
        @client.invoke(:getContainerList, node_uri, parameters: criteria)
      end

      # Returns list of containers. This method is a
      # convenience shortcut for the {#get_container_list} instance method.
      #
      # @see #get_container_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_container_list(node_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_container_list(node_uri, criteria)
      end

    end

  end
end
