require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Host.
    class Host < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/Host"

      # Creates a new instance of +Host+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(resource_group_uri, attributes)
      #   Creates a new host.
      #   @param resource_group_uri [String, UU::OS::UESURI] ResourceGroup URI.
      #   @param attributes [Hash] Host attributes.
      #   @option attributes [String] name Host name. May be +nil+.
      #   @option attributes [String] code Host code. May be +nil+.
      #   @option attributes [String] description Host description.
      #     May be +nil+.
      #   @option attributes [String] reinitStrategy Flag to enable
      #     reinit for the host. The only supported value at the moment
      #     is "SUPPORTED". May be +nil+.
      #   @option attributes [String] hostname Name of the physical host.
      #   @option attributes [Hash] totalCapacity Total capacity this host
      #     can provide for running nodes. Supported keys and value types:
      #     !{cpu: Numeric, mem: Fixnum, storage: Fixnum}. All keys must
      #     have values.
      #     * +:cpu (Fixnum)+ Number of virtual CPU cores.
      #     * +:mem (Fixnum)+ Memory limit in MB (Megabytes).
      #     * +:storage (Fixnum)+ Maximum storage limit in MB (Megabytes).
      #   @option attributes [Hash] labels Hash of labels to attach to a Host. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the created host.

      # @!method self.create(resource_group_uri, attributes)
      #   Creates a new host. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param resource_group_uri [String, UU::OS::UESURI] ResourceGroup URI.
      #   @param attributes [Hash] Host attributes.
      #   @return [UU::OS::UESURI] UESURI of the created host.

      # @!method get_attributes(host_uri)
      #   Gets the attributes of the host specified by host URI
      #   and returns its detail.
      #   @param host_uri [String, UU::OS::UESURI] Host URI.
      #   @return [Hash] Host attributes.

      # @!method self.get_attributes(host_uri)
      #   Gets the attributes of the host specified by host URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param host_uri [String, UU::OS::UESURI] Host URI.
      #   @return [Hash] Host attributes.

      # @!method set_attributes(host_uri, attributes)
      #   Sets attributes of a host.
      #
      #   @example Set labels
      #    require 'uu_operation_registry'
      #
      #    UU::OS::Security::Session.login('...')
      #
      #    labels = {
      #      :"uu.cloud.deployment-environment" => "group1"
      #    }
      #
      #    UU::OperationRegistry::Host.set_attributes(
      #      'ues:HOST:URI',
      #       labels: labels
      #    )
      #
      #   @param host_uri [String, UU::OS::UESURI] Host URI.
      #   @param attributes [Hash] Host attributes.
      #   @option attributes [String] name Host name. May be +nil+.
      #   @option attributes [String] code Host code. May be +nil+.
      #   @option attributes [String] description Host description.
      #     May be +nil+.
      #   @option attributes [String] reinitStrategy Flag to enable
      #     reinit for the host. The only supported value at the moment
      #     is "SUPPORTED". May be +nil+.
      #   @option attributes [String] hostname Name of the physical host.
      #     May be +nil+.
      #   @option attributes [String] state Host state. Value must be
      #     one from {UU::OperationRegistry::HostState}. Host must be in state ACTIVE for deployment.
      #     May be +nil+.
      #   @option attributes [Hash] totalCapacity Total capacity this host
      #     can provide for running nodes. May be +nil+. Supported keys and
      #     value types: !{cpu: Numeric, mem: Fixnum, storage: Fixnum}.
      #     All values are optional.
      #     * +:cpu (Fixnum)+ Number of virtual CPU cores.
      #     * +:mem (Fixnum)+ Memory limit in MB (Megabytes).
      #     * +:storage (Fixnum)+ Maximum storage limit in MB (Megabytes).
      #   @option attributes [Hash] labels Hash of labels to attach to a Host. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the updated host.

      # @!method self.set_attributes(host_uri, attributes)
      #   Sets attributes of a host. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param host_uri [String, UU::OS::UESURI] Host URI.
      #   @return [UU::OS::UESURI] UESURI of the updated host.

      # @!method delete(host_uri)
      #   Deletes the specified host from the registry.
      #   @param host_uri [String, UU::OS::UESURI] Host URI.
      #   @return [void]

      # @!method self.delete(host_uri)
      #   Deletes the specified host from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param host_uri [String, UU::OS::UESURI] Host URI.
      #   @return [void]

      # Returns a list of hosts in the given resource group. Command does
      # not return all hosts, only a sublist. The list of returned hosts
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param resource_group_uri [String, UU::OS::UESURI] ResourceGroup URI.
      # @param criteria [Hash] Criteria for listing hosts.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: 
      #   code, state, hostname, resourceGroupUri, totalCapacity, freeCapacity
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_host_list(resource_group_uri, criteria = {})
        @client.invoke(:getHostList, resource_group_uri, parameters: criteria)
      end

      # Returns list of hosts. This method is a
      # convenience shortcut for the {#get_host_list} instance method.
      #
      # @see #get_host_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_host_list(resource_group_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_host_list(resource_group_uri, criteria)
      end

    end

  end
end
