require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Region.
    class Region < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/Region"

      # Creates a new instance of +Region+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(cloud_uri, attributes)
      #   Creates a new region.
      #
      #   @example Create Region with the specified resources
      #    require 'uu_operation_registry'
      #
      #    UU::OS::Security::Session.login('...')
      #
      #    graylog_stream_url = '...'
      #    graylog_stream_id = '...'
      #    region_name = '...'
      #    region_code = '...'
      #
      #    UU::OperationRegistry::Region.create(
      #      'ues:CLOUD:URI',
      #      :name => region_name,
      #      :code => region_code,
      #      :resources => {
      #        :graylogSearchUrl => graylog_stream_url,
      #        :graylogStreamId => graylog_stream_id
      #      }
      #    )
      #
      #   @example Create Region with the specified env JSON
      #    require 'uu_operation_registry'
      #    require 'json'
      #
      #    UU::OS::Security::Session.login('...')
      #
      #    region_name = '...'
      #    region_code = '...'
      #    env = {
      #      "uuClientProperties" => {
      #        "uu.server-url" => "https://api.plus4u.net",
      #        "uu.c3.base_url" => "https://sys.plus4u.net",
      #        "uu.c3.append_routing_data" => "false",
      #        "uu.operation.registry.base_url" => "https://sys.plus4u.net",
      #        "uu.operation.registry.append_routing_data" => "false"
      #      },
      #      "gemrc" => {
      #        ":backtrace" => "false",
      #        ":benchmark" => "false",
      #        ":bulk_threshold" => "100",
      #        ":sources" => [
      #          "http://rubygems.org",
      #          "http://gems.plus4u.net"
      #        ],
      #        ":update_sources" => "true",
      #        ":verbose" => "false",
      #        "gem" => "--env-shebang --no-doc --no-ri --no-document"
      #      }
      #    }
      #
      #    UU::OperationRegistry::Region.create(
      #      'ues:CLOUD:URI',
      #      :name => region_name,
      #      :code => region_code,
      #      :env => env.to_json
      #    )
      #
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Region attributes.
      #   @option attributes [String] name Region name. May be +nil+.
      #   @option attributes [String] code Region code. May be +nil+.
      #   @option attributes [String] description Region description.
      #     May be +nil+.
      #   @option attributes [Hash] resources Resources associated with Region (e.g. Graylog configuration for uuLogstore).
      #     May be +nil+.
      #   @option attributes [String] env Env configuration JSON. Contains
      #     configuration (gemrc, uu-client.properties) for nodes deployed in the
      #     context of the Region.  May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the created region.

      # @!method self.create(cloud_uri, attributes)
      #   Creates a new region. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Region attributes.
      #   @return [UU::OS::UESURI] UESURI of the created region.

      # @!method get_attributes(region_uri)
      #   Gets the attributes of the region specified by region URI
      #   and returns its detail.
      #   @param region_uri [String, UU::OS::UESURI] Region URI.
      #   @return [Hash] Region attributes.

      # @!method self.get_attributes(region_uri)
      #   Gets the attributes of the region specified by region URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param region_uri [String, UU::OS::UESURI] Region URI.
      #   @return [Hash] Region attributes.

      # @!method set_attributes(region_uri, attributes)
      #   Sets attributes of a region.
      #
      #   @example Set Graylog configuration
      #    require 'uu_operation_registry'
      #
      #    UU::OS::Security::Session.login('...')
      #
      #    graylog_stream_url = '...'
      #    graylog_stream_id = '...'
      #    UU::OperationRegistry::Region.set_attributes(
      #      'ues:REGION:URI',
      #      :resources => {
      #        :graylogSearchUrl => graylog_stream_url,
      #        :graylogStreamId => graylog_stream_id
      #      }
      #    )
      #
      #   @example Set env JSON
      #    require 'uu_operation_registry'
      #    require 'json'
      #
      #    UU::OS::Security::Session.login('')
      #
      #    env = {
      #      "uuClientProperties" => {
      #        "uu.server-url" => "https://api.plus4u.net",
      #        "uu.c3.base_url" => "https://sys.plus4u.net",
      #        "uu.c3.append_routing_data" => "false",
      #        "uu.operation.registry.base_url" => "https://sys.plus4u.net",
      #        "uu.operation.registry.append_routing_data" => "false"
      #      },
      #      "gemrc" => {
      #        ":backtrace" => "false",
      #        ":benchmark" => "false",
      #        ":bulk_trashold" => "100",
      #        ":sources" => [
      #          "http://rubygems.org",
      #          "http://gems.plus4u.net"
      #        ],
      #        ":update_sources" => "true",
      #        ":verbose" => "false",
      #        "gem" => "--env-shebang --no-doc --no-ri --no-document"
      #      }
      #    }
      #
      #    UU::OperationRegistry::Region.set_attributes(
      #      'ues:REGION:URI',
      #      :env => env.to_json
      #    )
      #
      #   @param region_uri [String, UU::OS::UESURI] Region URI.
      #   @param attributes [Hash] Region attributes.
      #   @option attributes [String] name Region name. May be +nil+.
      #   @option attributes [String] code Region code. May be +nil+.
      #   @option attributes [String] description Region description.
      #     May be +nil+.
      #   @option attributes [Hash] resources Resources associated with Region (e.g. Graylog configuration for uuLogstore).
      #     May be +nil+.
      #   @option attributes [String] env Env configuration JSON. Containes configuration (gemrc, uu-client.properties)
      #     for nodes deployed in the context of the Region.  May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the updated region.

      # @!method self.set_attributes(region_uri, attributes)
      #   Sets attributes of a region. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param region_uri [String, UU::OS::UESURI] Region URI.
      #   @return [UU::OS::UESURI] UESURI of the updated region.

      # @!method delete(region_uri)
      #   Deletes the specified region from the registry.
      #   @param region_uri [String, UU::OS::UESURI] Region URI.
      #   @return [void]

      # @!method self.delete(region_uri)
      #   Deletes the specified region from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param region_uri [String, UU::OS::UESURI] Region URI.
      #   @return [void]

      # Returns a list of regions in the given cloud. Command does
      # not return all regions, only a sublist. The list of returned regions
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing regions.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, cloudUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_region_list(cloud_uri, criteria = {})
        @client.invoke(:getRegionList, cloud_uri, parameters: criteria)
      end

      # Returns list of regions. This method is a
      # convenience shortcut for the {#get_region_list} instance method.
      #
      # @see #get_region_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_region_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_region_list(cloud_uri, criteria)
      end

    end

  end
end
