require "uu/operation_registry"
require "uu/os/cmd/command_client"
require "uu/os/security/session"

module UU
  module OperationRegistry

    # Runtime Stack.
    class RuntimeStack < BaseMethods

      # Remote service path.
      # @private
      PATH = "uu-operation-registry/RuntimeStack"

      # Creates a new instance of +RuntimeStack+.
      #
      # @param session [UU::OS::Security::Session] Session to use for remote calls.
      def initialize(session = nil)
        super
      end

      # @!method create(cloud_uri, attributes)
      #   Creates a new runtime stack.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Runtime stack attributes.
      #   @option attributes [String] scalable Whether node images using this
      #     runtime stack can be scaled out/in. Value can be either +true+,
      #     +false+ or +nil+. Default value is +false+.
      #   @option attributes [String] name Runtime stack name. May be +nil+.
      #   @option attributes [String] code Runtime stack code. May be +nil+.
      #   @option attributes [String] description Runtime stack description.
      #     May be +nil+.
      #   @option attributes [String] alias Runtime stack alias. May be +nil+.
      #   @option attributes [UU::OS::UESURI] nodeImageUri Base image URI.
      #   @option attributes [String] buildRecipe Build recipe. May be +nil+.
      #   @option attributes [String] version Runtime stack version. Different versions can use different mechanisms
      #     of building node images. May be +nil+. If not specified, version +1.0+ is assumed. Supported versions:
      #     +1.0+, +2.0+.
      #   @option attributes [String] state Runtime stack state. Value must be
      #     one from {UU::OperationRegistry::RuntimeStackState}. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the created runtime stack.

      # @!method self.create(cloud_uri, attributes)
      #   Creates a new runtime stack. This method is a convenience shortcut for
      #   the {#create} instance method.
      #   @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      #   @param attributes [Hash] Runtime stack attributes.
      #   @return [UU::OS::UESURI] UESURI of the created runtime stack.

      # @!method get_attributes(runtime_stack_uri)
      #   Gets the attributes of the runtime stack specified by runtime stack URI
      #   and returns its detail.
      #   @param runtime_stack_uri [String, UU::OS::UESURI] Runtime stack URI.
      #   @return [Hash] Runtime stack attributes.

      # @!method self.get_attributes(runtime_stack_uri)
      #   Gets the attributes of the runtime stack specified by runtime stack URI
      #   and returns its detail. This method is a convenience shortcut for
      #   the {#get_attributes} instance method.
      #   @param runtime_stack_uri [String, UU::OS::UESURI] Runtime stack URI.
      #   @return [Hash] Runtime stack attributes.

      # @!method set_attributes(runtime_stack_uri, attributes)
      #   Sets attributes of a runtime stack.
      #   @param runtime_stack_uri [String, UU::OS::UESURI] Runtime stack URI.
      #   @param attributes [Hash] Runtime stack attributes.
      #   @option attributes [String] scalable Whether node images using this
      #     runtime stack can be scaled out/in. Value can be either +true+,
      #     +false+ or +nil+. Default value is +false+.
      #   @option attributes [String] name Runtime stack name. May be +nil+.
      #   @option attributes [String] code Runtime stack code. May be +nil+.
      #   @option attributes [String] description Runtime stack description.
      #     May be +nil+.
      #   @option attributes [String] alias Runtime stack alias. May be +nil+.
      #   @option attributes [String] buildRecipe Build recipe. May be +nil+.
      #   @option attributes [String] version Runtime stack version. Different versions can use different mechanisms
      #     of building node images. May be +nil+. Supported versions: +1.0+, +2.0+.
      #   @option attributes [String] state Runtime stack state. Value must be
      #     one from {UU::OperationRegistry::RuntimeStackState}. May be +nil+.
      #   @return [UU::OS::UESURI] UESURI of the updated runtime stack.

      # @!method self.set_attributes(runtime_stack_uri, attributes)
      #   Sets attributes of a runtime stack. This method is a convenience
      #   shortcut for the {#set_attributes} instance method.
      #   @param runtime_stack_uri [String, UU::OS::UESURI] Runtime stack URI.
      #   @return [UU::OS::UESURI] UESURI of the updated runtime stack.

      # @!method delete(runtime_stack_uri)
      #   Deletes the specified runtime stack from the registry.
      #   @param runtime_stack_uri [String, UU::OS::UESURI] Runtime stack URI.
      #   @return [void]

      # @!method self.delete(runtime_stack_uri)
      #   Deletes the specified runtime stack from the registry. This method is a
      #   convenience shortcut for the #{delete} instance method.
      #   @param runtime_stack_uri [String, UU::OS::UESURI] Runtime stack URI.
      #   @return [void]

      # Returns a list of runtime stacks in the given cloud. Command does
      # not return all runtime stacks, only a sublist. The list of returned runtime stacks
      # could be filtered using {https://docs.mongodb.org/manual/tutorial/query-documents/
      # Mongo Query}.
      #
      # @param cloud_uri [String, UU::OS::UESURI] Cloud URI.
      # @param criteria [Hash] Criteria for listing runtime stacks.
      # @option criteria [String] query Mongo Query to filter the result set.
      #   See {https://docs.mongodb.org/manual/tutorial/query-documents/ Mongo Query tutorial}.
      #   Allowed query filter document fields: code, nodeImageUri, cloudUri
      # @option criteria [Fixnum] pageIndex Result set page index.
      # @option criteria [Fixnum] pageSize Result set page size.
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def get_runtime_stack_list(cloud_uri, criteria = {})
        @client.invoke(:getRuntimeStackList, cloud_uri, parameters: criteria)
      end

      # Returns list of runtime stacks. This method is a
      # convenience shortcut for the {#get_runtime_stack_list} instance method.
      #
      # @see #get_runtime_stack_list
      # @return [UU::OS::Persistence::PagedResult<Hash>] Paged result set.
      def self.get_runtime_stack_list(cloud_uri, criteria = {})
        self.new(UU::OS::Security::Session.current_session).get_runtime_stack_list(cloud_uri, criteria)
      end

    end

  end
end
